@@ -41,6 +41,16 @@
 #include "as_except.h"          // Exception classes
 #include "as_tableitem.h"       // For our custom table items
 #include "as_generic.h"         // Generic classes
+#include "as_uuid.h"            // UUID classes
+
+// RPC headers
+#ifdef HAVE_ESOAP
+#include <easysoap/SOAP.h>
+#endif // HAVE_ESOAP
+#ifdef HAVE_MICO
+#include <coss/CosNaming.h>
+#include "asdb.h"               // CORBA stubs and skeletons
+#endif // HAVE_MICO
 
 // Icon pixel maps
 #include "as_gfx/cwlogo.xpm"    // static const char *s_kpcCwlogo_xpm[]
@@ -98,6 +108,8 @@ void Titraqform::pasteEntry(void)
         // Do basic data validation to warn against missing fields
         for (int nIter = 0; nIter < nRows; nIter++)
             this->validateData(m_pMaintable->currentRow() + nIter, 0);
+
+        updEdit(m_pMaintable->currentRow()); // Reflect in the update controls
     }
 }
 
@@ -109,6 +121,7 @@ void Titraqform::addEntry(int nRows)
     QTableSelection Select; // Highlighted text
     int nTotal = 0;         // Total row select
     int nCurrent = 0;       // Current row
+    std::auto_ptr<AS::Uuid> pGuid(new AS::Uuid); // For GUID production
 
     // Decide how many rows to add
     Select = m_pMaintable->selection(0);
@@ -131,9 +144,10 @@ void Titraqform::addEntry(int nRows)
             m_pMaintable->setText(Select.topRow() + nIter, TITRAQ_IDXSTATUS, QString(QChar('W')));
             m_pMaintable->setPixmap(Select.topRow() + nIter, TITRAQ_IDXSTATUS, QPixmap(s_kpcStatwarn_xpm));
             m_pMaintable->setText(Select.topRow() + nIter, TITRAQ_IDXUSER, m_pPrefs->getString(TITRAQ_PREFUSER, TITRAQ_DEFUSER));
-            m_pMaintable->setText(Select.topRow() + nIter, TITRAQ_IDXGUID, ".");
-            m_pMaintable->setText(Select.topRow() + nIter, TITRAQ_IDXCRC, ".");
-            m_pMaintable->setText(Select.topRow() + nIter, TITRAQ_IDXREV, ".");
+            pGuid->genId();
+            m_pMaintable->setText(Select.topRow() + nIter, TITRAQ_IDXGUID, pGuid->getString());
+            m_pMaintable->setText(Select.topRow() + nIter, TITRAQ_IDXCRC, "0");
+            m_pMaintable->setText(Select.topRow() + nIter, TITRAQ_IDXREV, "0");
             m_pMaintable->setText(Select.topRow() + nIter, TITRAQ_IDXDATE, QDate::currentDate().toString("yyyy.MM.dd"));
             m_pMaintable->setText(Select.topRow() + nIter, TITRAQ_IDXSTART, "00:00");
             m_pMaintable->setText(Select.topRow() + nIter, TITRAQ_IDXFINISH, "00:00");
@@ -153,9 +167,10 @@ void Titraqform::addEntry(int nRows)
             m_pMaintable->setText(Select.bottomRow() + nIter, TITRAQ_IDXSTATUS, QString(QChar('W')));
             m_pMaintable->setPixmap(Select.bottomRow() + nIter, TITRAQ_IDXSTATUS, QPixmap(s_kpcStatwarn_xpm));
             m_pMaintable->setText(Select.bottomRow() + nIter, TITRAQ_IDXUSER, m_pPrefs->getString(TITRAQ_PREFUSER, TITRAQ_DEFUSER));
-            m_pMaintable->setText(Select.bottomRow() + nIter, TITRAQ_IDXGUID, ".");
-            m_pMaintable->setText(Select.bottomRow() + nIter, TITRAQ_IDXCRC, ".");
-            m_pMaintable->setText(Select.bottomRow() + nIter, TITRAQ_IDXREV, ".");
+            pGuid->genId();
+            m_pMaintable->setText(Select.bottomRow() + nIter, TITRAQ_IDXGUID, pGuid->getString());
+            m_pMaintable->setText(Select.bottomRow() + nIter, TITRAQ_IDXCRC, "0");
+            m_pMaintable->setText(Select.bottomRow() + nIter, TITRAQ_IDXREV, "0");
             m_pMaintable->setText(Select.bottomRow() + nIter, TITRAQ_IDXDATE, QDate::currentDate().toString("yyyy.MM.dd"));
             m_pMaintable->setText(Select.bottomRow() + nIter, TITRAQ_IDXSTART, "00:00");
             m_pMaintable->setText(Select.bottomRow() + nIter, TITRAQ_IDXFINISH, "00:00");
@@ -169,15 +184,9 @@ void Titraqform::addEntry(int nRows)
     for (int nIter = m_pMaintable->currentRow(); nIter < m_pMaintable->numRows(); nIter++)
         m_pMaintable->setText(nIter, TITRAQ_IDXLINE, QString::number(nIter).rightJustify(4, QChar('0')));
 
+    updEdit(m_pMaintable->currentRow()); // Reflect in the update controls
     m_pStatusedit->setPixmap(QPixmap(s_kpcStatwarn_xpm)); // Show pixmap
 
-//    char szGuitext[37];
-//    uuid_t Guid;
-//
-//    uuid_generate(Guid);
-//    uuid_unparse(Guid, szGuitext);
-//    m_pMaintable->setText(m_pMaintable->currentRow(), TITRAQ_IDXGUID, szGuitext);
-
     m_pMaintable->setUpdatesEnabled(true);  // Turn updates back on
     m_pMaintable->repaintContents(true);    // Do a general repaint of table
 }
@@ -262,7 +271,7 @@ void Titraqform::newDoc(void)
     this->setCaption(trUtf8("No file name"));
     m_pStatbar->message(trUtf8("New document"), 4000);
     this->setFilename("");
-    this->updEdit(0, 0);                // Update edit controls
+    this->updEdit(0);                   // Update edit controls
 }
 
 //
@@ -1137,6 +1146,143 @@ void Titraqform::showRemarkcol(void)
     this->updSizes(TITRAQ_IDXALLCTRLS, 0, 0); // Update size of this and next
 }
 
+//
+// Generate a local formatted report
+//
+void Titraqform::genReport(void)
+{
+    Prototype Unimp;
+    Unimp.doMbox();
+}
+
+//
+// Syncronize data with server using IIOP
+//
+void Titraqform::syncIiop(void)
+{
+#ifdef HAVE_MICO
+    int nNada = 0;          // Some false parameters
+    char **ppcNada = NULL;  // to fake out the ORB
+    CORBA::ORB_var Orb;     // The ORB
+
+    CORBA::Object_var Nameobj;      // Name service reference
+    CosNaming::NamingContext_var Namectx; // COSS ns context
+    CosNaming::Name Cosname;        // Our requested obj name
+
+    CORBA::Object_var Objcaster;    // Generic CORBA object
+    Asdatabase_var Asdbase;         // Casted object to ASDB
+    Astuple Singlerow;              // A single row of AS data
+
+    try {
+        // Initialization of the ORB and COSS naming service
+        Orb = CORBA::ORB_init(nNada, ppcNada, "mico-local-orb");
+        Nameobj = Orb->resolve_initial_references("NameService");
+        Namectx = CosNaming::NamingContext::_narrow(Nameobj);
+        if (CORBA::is_nil(Namectx)) { // Verify sanity
+            m_pStatbar->message(trUtf8("Could not find the COSS naming service"));
+            qWarning("Could not find the COSS naming service\n");
+        }
+
+        // Prepare the COSS name request
+        Cosname.length(1);
+        Cosname[0].id = CORBA::string_dup("Asdatabase");
+        Cosname[0].kind = CORBA::string_dup("");
+
+        try { // Resolve to a CORBA object
+            Objcaster = Namectx->resolve(Cosname);
+        }
+        catch (CosNaming::NamingContext::NotFound &Cossex) {
+            m_pStatbar->message(trUtf8("NotFound exception thrown"));
+            qWarning("NotFound exception thrown\n");
+            return;
+        }
+        catch (CosNaming::NamingContext::CannotProceed &Cossex) {
+            m_pStatbar->message(trUtf8("CannotProceed exception thrown"));
+            qWarning("CannotProceed exception thrown\n");
+            return;
+        }
+        catch (CosNaming::NamingContext::InvalidName &Cossex) {
+            m_pStatbar->message(trUtf8("InvalidName exception thrown"));
+            qWarning("InvalidName exception thrown\n");
+            return;
+        }
+
+        // Cast the generic CORBA object to a AS database type
+        Asdbase = Asdatabase::_narrow(Objcaster);
+        if (CORBA::is_nil(Asdbase)) { // Verify sanity
+            m_pStatbar->message(trUtf8("Could not find the AS database"));
+            qWarning("Could not find the AS database\n");
+            return;
+        }
+
+        // Open an account object on the remote server
+        Account_var Account = Asdbase->Open("/tmp/corbadb.txt"); // Security problem!
+        if (CORBA::is_nil(Account)) { // Verify sanity
+            m_pStatbar->message(trUtf8("Could not create an account object on the server"));
+            qWarning("Could not create an account object on the server\n");
+            return;
+        }
+
+        // Fill an account object to marshall and transmit
+        int nLastrow = m_pMaintable->numRows() - 1;
+        Singlerow.nDate = m_pMaintable->text(nLastrow, TITRAQ_IDXDATE).toInt();
+        Singlerow.nTime = m_pMaintable->text(nLastrow, TITRAQ_IDXSTART).toInt();
+        Singlerow.szName = CORBA::string_dup(m_pMaintable->text(nLastrow, TITRAQ_IDXUSER));
+        Singlerow.szTask = CORBA::string_dup(m_pMaintable->text(nLastrow, TITRAQ_IDXTASK));
+        Singlerow.szRemark = CORBA::string_dup(m_pMaintable->text(nLastrow, TITRAQ_IDXREMARK));
+
+        Account->Log(Singlerow); // Finally transmit to server
+        m_pStatbar->message(trUtf8("Successful transmission of accounting data"));
+    }
+    catch (const CORBA::Exception &Corbex) {
+        m_pStatbar->message(trUtf8("Caught CORBA exception: %1").arg(Corbex._repoid()));
+        qWarning("Caught CORBA exception: %s\n", Corbex._repoid());
+    }
+    catch (...) {
+        qWarning("Caught unknown exception\n");
+    }
+#endif // HAVE_MICO
+}
+
+//
+// Syncronize data with server using SOAP
+//
+void Titraqform::syncSoap(void)
+{
+#ifdef HAVE_ESOAP
+    USING_EASYSOAP_NAMESPACE
+
+    try {
+        const char *pSpace    = "http://soap.europalab.com/asdb";
+        const char *pEndpoint = "http://www.europalab.com/cgi-bin/asdbserv";
+        SOAPProxy Proxy(pEndpoint);
+        SOAPMethod Logmeth("Log", pSpace);  // SOAP remote method name
+        SOAPString Clistr;                  // Outgoing parameter to marshall
+        int nCrc;                           // SOAP unmarshalled return value
+
+        // A quasi diff summary to transmit to the server
+        QString Syncthis;
+        int nLastrow = m_pMaintable->numRows() - 1;
+        Syncthis += m_pMaintable->text(nLastrow, 0);
+        for (int nIter = 1; nIter < TITRAQ_IDXTAIL; nIter++)
+            Syncthis += ' ' + m_pMaintable->text(nLastrow, nIter);
+
+        Clistr = Syncthis;                          // Build RPC parameter
+        Logmeth.AddParameter("Tuple") << Clistr;    // Prepare for marshalling
+        const SOAPResponse &Logresp = Proxy.Execute(Logmeth);
+        Logresp.GetReturnValue() >> nCrc;
+        m_pStatbar->message(trUtf8("Successful transmission, CRC returned %1").arg(nCrc));
+    }
+    catch (SOAPException& Soapex) { // Announce the exception we received
+        m_pStatbar->message(trUtf8("Caught SOAP exception: %1").arg(Soapex.What().Str()));
+        qDebug("Caught SOAP exception: %s\n", Soapex.What().Str());
+    }
+    catch (...) {
+        qDebug("Caught unknown exception\n");
+    }
+#endif // HAVE_ESOAP
+}
+
 //
 // Save user preferences
 //
